package vue;
import java.util.Scanner;

import classes_metiers.Coup;
import classes_metiers.Symbole;
import controleur.InterfaceControleurMorpion;
import modele.InterfaceModeleMorpion;
import modele.ModeleMorpion;
import modele.Observateur; 

/**
 * @author Dominique Bouthinon, révision B. KHAFIF
 * 
 */
public class VueMorpionTextuel implements InterfaceVueMorpion, Observateur
{
	
	private InterfaceControleurMorpion controleur ;
	private InterfaceModeleMorpion modele ;		
	
	public VueMorpionTextuel(InterfaceControleurMorpion unControleur, 
							  InterfaceModeleMorpion unModelePoint)
	{
		this.controleur = unControleur ;
		this.modele     = unModelePoint ;
		this.modele.enregistrerObservateur(this) ;
	}
	
	
	

	/********
	 * Interface Observable
	 */
	
		/**
		 *  la vue est informee que le modele a change
		 */
	public void actualiser()
	{
		this.afficherGrille()   ;
	}
	
	
	
	/********
	 * Interface InterfaceVueMorpion
	 */
	
	/**
	 * Active la vue
	 */
	public void activerVue()
	{
		int choix ;
		
		this.afficherGrille() ;
		do
		{
			this.afficherJoueurActif() ;
			this.afficherMenu();
			choix = saisirChoix() ;
			this.traiterChoix(choix) ;
		}
		while(choix != VueMorpionTextuel.SORTIE_PROGRAMME &&
			  !this.modele.partieFinie()) ;
	}
	
	
	
	/**
	 * Affiche le joueur actif
	 */
	public void afficherJoueurActif()
	{
		System.out.println("\nc'est a " + this.modele.getJoueurActif() + " de jouer : ") ;
	}
	
	
	/**
	 * Affiche le coup qui vient d'être joué
	 */
	public void afficherCoupJoue()
	{
		Coup c = this.modele.getCoupPropose() ;
		
		System.out.println("Coup joue : ") ;
		System.out.println("ligne    = " + c.getLigne())   ;
		System.out.println("colonne  = " + c.getColonne()) ;
		System.out.println("valeur   = " + c.getSymbole()) ;
	}
	
	
	
	/**
	 * affiche la grille actuelle 
	 */
	public void afficherGrille()
	{
		Symbole[][] grille = this.modele.getGrille() ;
		
		
		System.out.println("Grille courante : \n") ;
	
		this.afficherNumeroColonnes() ;
		
		
		for (int ligne = 0 ; ligne < grille.length ; ligne++)
		{
			System.out.print(ligne + " ") ;
			
			for (int colonne = 0 ; colonne < grille[ligne].length ; colonne++)
			{
				System.out.print("|" + grille[ligne][colonne].getLibelle()) ;	
			}
			
			System.out.println("|") ;
		}
	}
	
	
	/**
	 * Affiche les inforamtion de fin de partie
	 */
	public void afficherFinPartie()
	{
		System.out.println("\n") ;
		System.out.println("***** la partie est finie ******* ") ;
		System.out.println("le gagnant est " + 
		                    VueMorpionTextuel.this.modele.getJoueurActif()) ;
		
		System.out.println("\n") ;
	}
	
	

	/**
	 * Affiche les informations concernant le coup erroné
	 */
	public void afficherErreurCoupPropose()
	{
		int  ligne      = this.modele.getCoupPropose().getLigne()   ;
		int  colonne    = this.modele.getCoupPropose().getColonne() ;
		Symbole valeur  = this.modele.getCoupPropose().getSymbole()  ;
		
		
		System.out.println("!!! ce coup : ligne = "+ ligne + " colonne = " + colonne + 
				           " valeur = " + valeur + " est invalide !!!!!!") ;
	}
	
	
	/**
	 * Intéraction pour la saisie de la ligne
	 */
	public int saisirLigne() 
	{
		Scanner s = new Scanner(System.in) ;
		
		System.out.println("ligne ?") ;
		return  s.nextInt();
		
		// ne pas faire s.close() java bug
	}
	
	/**
	 * Intéraction pour la saisie de la colonne
	 */
	public int saisirColonne() 
	{
		Scanner s = new Scanner(System.in) ;
		
		System.out.println("colonne ?") ;
		return s.nextInt();
		
		// ne pas faire s.close java bug
	}
	
	
	
	/**
	 * La vue demande au contrôleur de gerer la saisie d'un coup
	 */
	public void saisirCoup()
	{
		this.controleur.gererSaisirCoup();
	}
	
	
	
	
	
	// *********************** MENU PRINCIPAL *******************
	

	public static final int SAISIR_COUP      = 1 ;
	public static final int SORTIE_PROGRAMME = 2 ;
	
	
	
	
	private void afficherMenu()
	{
		System.out.println("") ;
		System.out.println("1 - entrer un coup") ;
		System.out.println("2 - sortir du jeu") ;
		System.out.println("\n") ;
	}
	
	
	
	private int saisirChoix()
	{
		Scanner s = new Scanner(System.in) ;
		
		int choix ;
		boolean ok = false ;
		
		do
		{
			System.out.println("entrer votre choix (puis Entree)") ;
			choix = s.nextInt();
			
			ok = (choix >= VueMorpionTextuel.SAISIR_COUP && 
				  choix <= VueMorpionTextuel.SORTIE_PROGRAMME) ;
			
			if(!ok)
				VueMorpionTextuel.afficherErreurChoix() ;
		}
		while(!ok) ;
		
		
		return choix ;
	}
	
	
	private void traiterChoix(int choix)
	{
		switch(choix)
		{
			case VueMorpionTextuel.SAISIR_COUP 	     : this.saisirCoup()            ; break ;
			case VueMorpionTextuel.SORTIE_PROGRAMME  : VueMorpionTextuel.afficherMessageSortie() ; 
		}
	}
	
	
	
	// **************************** AFFICHAGE ************************
	

	 
	private void afficherNumeroColonnes()
	{
		System.out.print("   ") ;
		for(int colonne = 0 ; colonne < this.modele.getGrille().length ; colonne++)
		{
			System.out.print(colonne + " ") ;
		}
		
		System.out.println("") ;
	}
	
	
			// ************************* MESSAGES ************************	
	
	
	
	public static void afficherMessageSortie()
	{
		System.out.println("\n\n au revoir ! \n \n") ;
	}
	
	
	public static void afficherErreurChoix()
	{
		System.out.println("!!!choix invalide !!! ") ;
	}
	
	
	
	// **************************************** SAISIE ***********************************
	
	

	
}
// fin classe VueMorpionTextuel
